import type { ModalMethods, ModalProps, ReturnMethods, UseModalInnerReturnType, UseModalReturnType } from '../typing'
import { isProdMode } from '@/utils/env'
import { isFunction } from '@/utils/is'
import { error } from '@/utils/log'
import { tryOnUnmounted } from '@vueuse/core'
import { isEqual } from 'lodash-es'
import { computed, getCurrentInstance, nextTick, onUnmounted, reactive, ref, toRaw, unref, watchEffect } from 'vue'

const dataTransfer = reactive<any>({})

const openData = reactive<{ [key: number]: boolean }>({})

/**
 * @description: Applicable to independent modal and call outside
 */
export function useModal<D = any>(): UseModalReturnType<D> {
  const modal = ref<Nullable<ModalMethods>>(null)
  const loaded = ref<Nullable<boolean>>(false)
  const uid = ref<number>(0)

  function register(modalMethod: ModalMethods, uuid: number) {
    if (!getCurrentInstance())
      throw new Error('useModal() can only be used inside setup() or functional components!')

    uid.value = uuid
    isProdMode()
    && onUnmounted(() => {
      modal.value = null
      loaded.value = false
      dataTransfer[unref(uid)] = null
    })
    if (unref(loaded) && isProdMode() && modalMethod === unref(modal))
      return

    modal.value = modalMethod
    loaded.value = true
    modalMethod.emitOpen = (open: boolean, uid: number) => {
      openData[uid] = open
    }
  }

  const getInstance = () => {
    const instance = unref(modal)
    if (!instance)
      error('useModal instance is undefined!')

    return instance
  }

  const methods: ReturnMethods<D> = {
    setModalProps: (props: Partial<ModalProps>): void => {
      getInstance()?.setModalProps(props)
    },

    getOpen: computed((): boolean => {
      return openData[~~unref(uid)]
    }),

    redoModalHeight: () => {
      getInstance()?.redoModalHeight?.()
    },

    openModal: (open = true, data?: D, openOnSet = true): void => {
      getInstance()?.setModalProps({
        open,
      })

      if (!data)
        return
      const id = unref(uid)
      if (openOnSet) {
        dataTransfer[id] = null
        dataTransfer[id] = toRaw(data)
        return
      }
      const equal = isEqual(toRaw(dataTransfer[id]), toRaw(data))
      if (!equal)
        dataTransfer[id] = toRaw(data)
    },

    closeModal: () => {
      getInstance()?.setModalProps({ open: false })
    },
  }
  return [register, methods]
}

export function useModalInner(callbackFn?: Fn): UseModalInnerReturnType {
  const modalInstanceRef = ref<Nullable<ModalMethods>>(null)
  const currentInstance = getCurrentInstance()
  const uidRef = ref<number>(0)

  const getInstance = () => {
    const instance = unref(modalInstanceRef)
    if (!instance)
      error('useModalInner instance is undefined!')

    return instance
  }

  const register = (modalInstance: ModalMethods, uuid: number) => {
    isProdMode()
    && tryOnUnmounted(() => {
      modalInstanceRef.value = null
    })
    uidRef.value = uuid
    modalInstanceRef.value = modalInstance
    currentInstance?.emit('register', modalInstance, uuid)
  }

  watchEffect(() => {
    const data = dataTransfer[unref(uidRef)]
    if (!data)
      return
    if (!callbackFn || !isFunction(callbackFn))
      return
    nextTick(() => {
      callbackFn(data)
    })
  })

  return [
    register,
    {
      changeLoading: (loading = true) => {
        getInstance()?.setModalProps({ loading })
      },
      getOpen: computed((): boolean => {
        return openData[~~unref(uidRef)]
      }),

      changeOkLoading: (loading = true) => {
        getInstance()?.setModalProps({ confirmLoading: loading })
      },

      closeModal: () => {
        getInstance()?.setModalProps({ open: false })
      },

      setModalProps: (props: Partial<ModalProps>) => {
        getInstance()?.setModalProps(props)
      },

      redoModalHeight: () => {
        const callRedo = getInstance()?.redoModalHeight
        callRedo && callRedo()
      },
    },
  ]
}
