import type { AppRouteRecordRaw, Menu } from '@/router/types'
import { PageEnum } from '@/enums/pageEnum'

import { useI18n } from '@/hooks/web/useI18n'
import { useMessage } from '@/hooks/web/useMessage'
import { transformRouteToMenu } from '@/router/helper/menuHelper'
import { flatMultiLevelRoutes, transformObjToRoute } from '@/router/helper/routeHelper'
import { PAGE_NOT_FOUND_ROUTE } from '@/router/routes/basic'
import profile from '@/router/routes/modules/profile'
import { store } from '@/store'
import { filter } from '@/utils/helper/treeHelper'
import { defineStore } from 'pinia'
import { toRaw } from 'vue'
import { useUserStore } from './user'

interface PermissionState {
  // Permission code list
  // 权限代码列表
  permCodeList: string[] | number[]
  // Whether the route has been dynamically added
  // 路由是否动态添加
  isDynamicAddedRoute: boolean
  // To trigger a menu update
  // 触发菜单更新
  lastBuildMenuTime: number
  // Backstage menu list
  // 后台菜单列表
  backMenuList: Menu[]
}

export const usePermissionStore = defineStore('app-permission', {
  state: (): PermissionState => ({
    // 权限代码列表
    permCodeList: [],
    // Whether the route has been dynamically added
    // 路由是否动态添加
    isDynamicAddedRoute: false,
    // To trigger a menu update
    // 触发菜单更新
    lastBuildMenuTime: 0,
    // Backstage menu list
    // 后台菜单列表
    backMenuList: [],
  }),
  getters: {
    getPermCodeList(state): string[] | number[] {
      return state.permCodeList
    },
    getBackMenuList(state): Menu[] {
      return state.backMenuList
    },
    getLastBuildMenuTime(state): number {
      return state.lastBuildMenuTime
    },
    getIsDynamicAddedRoute(state): boolean {
      return state.isDynamicAddedRoute
    },
  },
  actions: {
    setPermCodeList(codeList: string[]) {
      this.permCodeList = codeList
    },

    setBackMenuList(list: Menu[]) {
      this.backMenuList = list
      list?.length > 0 && this.setLastBuildMenuTime()
    },

    setLastBuildMenuTime() {
      this.lastBuildMenuTime = new Date().getTime()
    },

    setDynamicAddedRoute(added: boolean) {
      this.isDynamicAddedRoute = added
    },
    resetState(): void {
      this.isDynamicAddedRoute = false
      this.permCodeList = []
      this.backMenuList = []
      this.lastBuildMenuTime = 0
    },
    changePermissionCode(codeList: string[]) {
      this.setPermCodeList(codeList)
    },

    // 构建路由
    async buildRoutesAction(): Promise<AppRouteRecordRaw[]> {
      const { t } = useI18n()
      const userStore = useUserStore()
      let routes: AppRouteRecordRaw[] = []
      const userInfo = toRaw(userStore.getUserInfo) || {}

      /**
       * @description 移除 ignoreRoute 的路由
       */
      const routeRemoveIgnoreFilter = (route: AppRouteRecordRaw) => {
        const { meta } = route
        const { ignoreRoute } = meta || {}
        return !ignoreRoute
      }

      /**
       * @description 根据设置的首页path，修正routes中的affix标记（固定首页）
       */
      const patchHomeAffix = (routes: AppRouteRecordRaw[]) => {
        if (!routes || routes.length === 0)
          return
        let homePath: string = PageEnum.BASE_HOME

        function patcher(routes: AppRouteRecordRaw[], parentPath = '') {
          if (parentPath)
            parentPath = `${parentPath}/`
          routes.forEach((route: AppRouteRecordRaw) => {
            const { path, children, redirect } = route
            const currentPath = path.startsWith('/') ? path : parentPath + path
            if (currentPath === homePath) {
              if (redirect) {
                homePath = route.redirect! as string
              }
              else {
                route.meta = Object.assign({}, route.meta, { affix: true })
                throw new Error('end')
              }
            }
            children && children.length > 0 && patcher(children, currentPath)
          })
        }

        try {
          patcher(routes)
        }
        catch {
          // 已处理完毕跳出循环
        }
      }

      const { createMessage } = useMessage()
      createMessage.loading({ content: t('sys.app.menuLoading'), duration: 1 })

      // Dynamically introduce components
      let routeList = transformObjToRoute(userInfo.menus)

      //  Background routing to menu structure
      const backMenuList = transformRouteToMenu([profile, ...routeList])
      this.setBackMenuList(backMenuList)

      // remove meta.ignoreRoute item
      routeList = filter(routeList, routeRemoveIgnoreFilter)
      routeList = routeList.filter(routeRemoveIgnoreFilter)
      routeList = flatMultiLevelRoutes(routeList)

      routes = [PAGE_NOT_FOUND_ROUTE, profile, ...routeList]
      patchHomeAffix(routes)
      return routes
    },
  },
})

// Need to be used outside the setup
// 需要在设置之外使用
export function usePermissionStoreWithOut() {
  return usePermissionStore(store)
}
