import type { __MenuItem } from '@/api/base/user'
import type { LoginParams, User, UserInfo } from '@/api/base/user/types'
import type { MenuItem } from '@/api/system/menu/types'
import type { RoleEnum } from '@/enums/roleEnum'

import type { ErrorMessageMode } from '@/types/axios'
import type { RouteRecordRaw } from 'vue-router'
import { doLogout, getUserButtons, getUserRouters, loginApi } from '@/api/base/user'
import { ACCESS_TOKEN_KEY, REFRESH_TOKEN_KEY, ROLES_KEY, USER_INFO_KEY } from '@/enums/cacheEnum'
import { PageEnum } from '@/enums/pageEnum'
import { useI18n } from '@/hooks/web/useI18n'
import { useMessage } from '@/hooks/web/useMessage'
import { router } from '@/router'
import { PAGE_NOT_FOUND_ROUTE } from '@/router/routes/basic'
import { store } from '@/store'
import { usePermissionStore } from '@/store/modules/permission'
import { getAuthCache, setAuthCache, setTenantId } from '@/utils/auth'
import { defineStore } from 'pinia'
import { h } from 'vue'

interface UserState {
  userInfo: Nullable<UserInfo>
  accessToken?: string
  refreshToken?: string
  roleList: RoleEnum[]
  sessionTimeout?: boolean
  lastUpdateTime: number
}

export const useUserStore = defineStore('app-user', {
  state: (): UserState => ({
    // user info
    userInfo: null,
    // token
    accessToken: undefined,
    refreshToken: undefined,
    // roleList
    roleList: [],
    // Whether the login expired
    sessionTimeout: false,
    // Last fetch time
    lastUpdateTime: 0,
  }),
  getters: {
    getUserInfo(state): UserInfo {
      return state.userInfo || getAuthCache<UserInfo>(USER_INFO_KEY) || {}
    },
    getAccessToken(state): string {
      return state.accessToken || getAuthCache<string>(ACCESS_TOKEN_KEY)
    },
    getRefreshToken(state): string {
      return state.refreshToken || getAuthCache<string>(REFRESH_TOKEN_KEY)
    },
    getRoleList(state): RoleEnum[] {
      return state.roleList.length > 0 ? state.roleList : getAuthCache<RoleEnum[]>(ROLES_KEY)
    },
    getSessionTimeout(state): boolean {
      return !!state.sessionTimeout
    },
    getLastUpdateTime(state): number {
      return state.lastUpdateTime
    },
  },
  actions: {
    setAccessToken(info: string | undefined) {
      this.accessToken = info || '' // for null or undefined value
      setAuthCache(ACCESS_TOKEN_KEY, info)
    },
    setRefreshToken(info: string | undefined) {
      this.refreshToken = info || '' // for null or undefined value
      setAuthCache(REFRESH_TOKEN_KEY, info)
    },
    setRoleList(roleList: RoleEnum[]) {
      this.roleList = roleList
      setAuthCache(ROLES_KEY, roleList)
    },
    setUserInfo(info: UserInfo | null) {
      this.userInfo = info
      this.lastUpdateTime = new Date().getTime()
      setAuthCache(USER_INFO_KEY, info)
    },
    setSessionTimeout(flag: boolean) {
      this.sessionTimeout = flag
    },
    resetState() {
      this.userInfo = null
      this.accessToken = ''
      this.roleList = []
      this.sessionTimeout = false
    },
    /**
     * @description: login
     */
    async login(
      params: LoginParams & {
        goHome?: boolean
        mode?: ErrorMessageMode
      },
    ): Promise<UserInfo | null> {
      try {
        const { goHome = true, mode, ...loginParams } = params
        const data = await loginApi(loginParams, mode)
        const { access_token, refresh_token, ...user } = data

        // save token
        this.setAccessToken(access_token)
        this.setRefreshToken(refresh_token)
        return this.afterLoginAction(goHome, user)
      }
      catch (error) {
        return Promise.reject(error)
      }
    },
    async afterLoginAction(goHome?: boolean, user?: User): Promise<UserInfo | null> {
      if (!this.getAccessToken)
        return null
      // get user info
      const userInfo = await this.getUserInfoAction(user!)

      const sessionTimeout = this.sessionTimeout
      if (sessionTimeout) {
        this.setSessionTimeout(false)
      }
      else {
        const permissionStore = usePermissionStore()
        if (!permissionStore.isDynamicAddedRoute) {
          const routes = await permissionStore.buildRoutesAction()
          routes.forEach((route) => {
            try {
              router.addRoute(route as unknown as RouteRecordRaw)
            }
            catch {}
          })
          router.addRoute(PAGE_NOT_FOUND_ROUTE as unknown as RouteRecordRaw)
          permissionStore.setDynamicAddedRoute(true)
        }
        goHome && (await router.replace(PageEnum.BASE_HOME))
      }
      return userInfo
    },
    async getUserInfoAction(user: User): Promise<UserInfo | null> {
      if (!this.getAccessToken)
        return null

      function normalizeMenus(__menus: __MenuItem[]): MenuItem[] {
        const menus: MenuItem[] = []
        for (let i = 0; i < __menus.length; i++) {
          const __menu = __menus[i]
          // componentName = /system/menu       -> SystemMenu
          //                 /system/menu/index -> SystemMenu
          const componentName = __menu.path
            .replace('/index', '')
            .split('/')
            .filter(Boolean)
            .map(i => `${i[0].toUpperCase()}${i.slice(1)}`)
            .join('')

          const component = __menu.path.replaceAll(/\/:[\s\S]*/g, '')

          let isPadIndexExtension = false
          if (!component.endsWith('index') && !/\/:/.test(__menu.path))
            isPadIndexExtension = true

          menus.push({
            id: __menu.id,
            parentId: __menu.parentId,
            name: __menu.name,
            component: `${component}${isPadIndexExtension ? '/index' : ''}.vue`,
            componentName,
            path: __menu.path,
            icon: __menu.source,
            sort: __menu.sort,
            children: __menu.children ? normalizeMenus(__menu.children) : [],
            code: __menu.code,
            type: 1,
            visible: __menu.visible,
            keepAlive: __menu.isOpen === 2 ? 1 : 0,
          })
        }
        return menus
      }

      let userInfo: UserInfo | null = null
      try {
        const [menus, buttons] = await Promise.all([
          getUserRouters(),
          getUserButtons(),
        ])
        userInfo = { menus: normalizeMenus(menus), buttons, user }
      }
      catch {
        return null
      }
      setTenantId(userInfo.user.tenant_id)
      usePermissionStore().changePermissionCode(userInfo.buttons)
      this.setUserInfo(userInfo)
      return userInfo
    },
    /**
     * @description: logout
     */
    async logout(goLogin = false) {
      if (this.getAccessToken) {
        try {
          await doLogout()
        }
        catch {
          console.log('注销Token失败')
        }
      }
      this.setAccessToken(undefined)
      this.setSessionTimeout(false)
      this.setUserInfo(null)
      goLogin && router.push(PageEnum.BASE_LOGIN)
    },

    /**
     * @description: Confirm before logging out
     */
    confirmLoginOut() {
      const { createConfirm } = useMessage()
      const { t } = useI18n()
      createConfirm({
        iconType: 'warning',
        title: () => h('span', t('sys.app.logoutTip')),
        content: () => h('span', t('sys.app.logoutMessage')),
        onOk: async () => {
          await this.logout(true)
        },
      })
    },
  },
})

// Need to be used outside the setup
export function useUserStoreWithOut() {
  return useUserStore(store)
}
