import type { BasicColumn, FormSchema } from '@/components/Table'
import type { Ref } from 'vue'
import { getSystemDictionary } from '@/api/base/common'
import { getDeptTree } from '@/api/system/dept'
import { getPostTree } from '@/api/system/post'
import { getRoleTree } from '@/api/system/role'
import { getAllTenants } from '@/api/system/tenant'
import { Avatar } from 'ant-design-vue'
import { h } from 'vue'

export const columns: BasicColumn[] = [
  {
    title: '登陆账号',
    dataIndex: 'account',
    width: 180,
  },
  {
    title: '所属租户',
    dataIndex: 'tenantName',
    width: 100,
  },
  {
    title: '用户姓名',
    dataIndex: 'realName',
    customRender({ record, value }) {
      return h('div', [
        h(
          Avatar,
          {
            src: /^https?/.test(record.avatar) ? record.avatar : undefined,
            size: 'small',
            style: { marginRight: '10px' },
          },
          () => value?.slice(0, 1),
        ),
        value,
      ])
    },
  },
  {
    title: '手机号码',
    dataIndex: 'phone',
    width: 120,
  },
  {
    title: '用户邮箱',
    dataIndex: 'email',
    width: 120,
  },
  {
    title: '所属角色',
    dataIndex: 'roleName',
    width: 120,
  },
  {
    title: '所属部门',
    dataIndex: 'deptName',
    width: 120,
  },
  {
    title: '用户平台',
    dataIndex: 'userTypeName',
    width: 120,
  },
]

export const searchFormSchema: FormSchema[] = [
  {
    label: '登陆账号',
    field: 'account',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '用户姓名',
    field: 'realName',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '用户平台',
    field: 'userType',
    component: 'ApiSelect',
    componentProps: {
      api: () => getSystemDictionary('user_type'),
    },
    colProps: { span: 6 },
  },
]

export function getFormSchema(isUpdate: Ref<boolean>): FormSchema[] {
  return [
    {
      field: 'id',
      show: false,
      component: 'Input',
    },
    {
      field: 'BaseInfo',
      label: '基本信息',
      component: 'Divider',
      componentProps: {
        plain: false,
      },
      colProps: {
        span: 24,
      },
    },
    {
      label: '所属租户',
      field: 'tenantId',
      required: true,
      component: 'ApiSelect',
      defaultValue: undefined,
      dynamicDisabled: () => isUpdate.value,
      componentProps({ formActionType, formModel }) {
        return {
          api: getAllTenants,
          valueField: 'tenantId',
          labelField: 'tenantName',
          onChange(value?: string) {
            function updateRole(treeData: { id: string, title: string }[]) {
              if (!isUpdate)
                formModel.roleId && formActionType.setFieldsValue({ roleId: undefined })

              formActionType.updateSchema({
                field: 'roleId',
                componentProps: {
                  treeData,
                },
              })
            }
            function updateDept(treeData: { id: string, title: string }[]) {
              if (!isUpdate)
                formModel.deptId && formActionType.setFieldsValue({ deptId: undefined })

              formActionType.updateSchema({
                field: 'deptId',
                componentProps: {
                  treeData,
                },
              })
            }
            function updatePost(treeData: { id: string, postName: string }[]) {
              if (!isUpdate)
                formModel.postId && formActionType.setFieldsValue({ postId: undefined })

              formActionType.updateSchema({
                field: 'postId',
                componentProps: {
                  treeData,
                },
              })
            }

            if (!value) {
              updateRole([])
              updateDept([])
              updatePost([])
              return
            }

            getRoleTree({ tenantId: value })
              .then((res) => {
                updateRole(res)
              })

            getDeptTree({ tenantId: value })
              .then((res) => {
                updateDept(res)
              })

            getPostTree({ tenantId: value })
              .then((res) => {
                updatePost(res)
              })
          },
        }
      },
      colProps: {
        span: 24,
      },
    },
    {
      label: '用户平台',
      field: 'userType',
      component: 'ApiSelect',
      required: true,
      componentProps: {
        api: () => getSystemDictionary('user_type'),
      },
    },
    {
      label: '账号',
      field: 'account',
      required: true,
      component: 'Input',
      ifShow: () => !isUpdate.value,
    },
    {
      label: '密码',
      field: 'password',
      required: true,
      component: 'InputPassword',
      ifShow: () => !isUpdate.value,
    },
    {
      label: '确认密码',
      field: 'confirmPassword',
      required: true,
      component: 'InputPassword',
      ifShow: () => !isUpdate.value,
      dynamicRules: ({ values }) => {
        return [
          {
            required: true,
            validator: (_, value) => {
              if (value !== values.password)
                // eslint-disable-next-line prefer-promise-reject-errors
                return Promise.reject('两次输入的密码不一致!')

              return Promise.resolve()
            },
          },
        ]
      },
    },
    {
      field: 'MoreInfo',
      label: '详细信息',
      component: 'Divider',
      componentProps: {
        plain: false,
      },
      colProps: {
        span: 24,
      },
    },
    {
      label: '用户昵称',
      field: 'name',
      component: 'Input',
      required: true,
    },
    {
      label: '用户姓名',
      field: 'realName',
      component: 'Input',
      required: true,
    },
    {
      label: '用户邮箱',
      field: 'email',
      component: 'Input',
      rules: [
        { type: 'email', message: '邮箱格式不正确' },
      ],
    },
    {
      label: '手机号码',
      field: 'mobile',
      component: 'InputNumber',
      componentProps: {
        controls: false,
        precision: 0,
      },
      rules: [
        { pattern: /^1[3-9]\d{9}$/, message: '手机号码格式不正确' },
      ],
    },
    {
      label: '用户性别',
      field: 'sex',
      component: 'Select',
      componentProps: {
        options: [
          {
            label: '男',
            value: 1,
          },
          {
            label: '女',
            value: 0,
          },
          {
            label: '未知',
            value: -1,
          },
        ],
      },
    },
    {
      label: '用户生日',
      field: 'birthday',
      component: 'DatePicker',
      componentProps: {
        style: {
          width: '100%',
        },
      },
    },
    {
      field: 'UserInfo',
      label: '职责信息',
      component: 'Divider',
      componentProps: {
        plain: false,
      },
      colProps: {
        span: 24,
      },
    },
    {
      label: '用户编号',
      field: 'code',
      component: 'Input',
    },
    {
      label: '所属部门',
      field: 'deptId',
      required: true,
      component: 'TreeSelect',
      componentProps: {
        treeData: [],
        fieldNames: {
          label: 'title',
          value: 'id',
        },
        multiple: true,
        treeCheckStrictly: false,
      },
    },
    {
      label: '所属角色',
      field: 'roleId',
      required: true,
      component: 'TreeSelect',
      componentProps: {
        treeData: [],
        fieldNames: {
          label: 'title',
          value: 'id',
        },
        multiple: true,
        treeCheckStrictly: false,
      },
    },
    {
      label: '所属岗位',
      field: 'postId',
      required: true,
      component: 'TreeSelect',
      componentProps: {
        treeData: [],
        fieldNames: {
          label: 'postName',
          value: 'id',
        },
        multiple: true,
        treeCheckStrictly: false,
      },
    },
  ]
}
